// DrawingPanel.java
// A scrollable, zoomable panel containing a drawing
import java.awt.*;
import java.io.*;
import Drawing;

public class DrawingPanel extends java.awt.Panel
{
	DrawingCanvas dcanvas;
	Panel zoompanel;
	Checkbox zoom1, zoom2, zoom3, zoom4;

	DrawingPanel(Drawing dr, boolean zm, boolean sc)
	{
	// Create components
	setLayout(new BorderLayout());
	add("Center",dcanvas = new DrawingCanvas(dr,sc));
	if (zm) {
		add("North",zoompanel = new Panel());
		zoompanel.setLayout(new FlowLayout());
		CheckboxGroup gr = new CheckboxGroup();
		zoompanel.add(new Label("Zoom"));
		zoompanel.add(zoom1 = new Checkbox("x 1/2",gr,false));
		zoompanel.add(zoom2 = new Checkbox("x 1",gr,true));
		zoompanel.add(zoom3 = new Checkbox("x 2",gr,false));
		zoompanel.add(zoom4 = new Checkbox("x 4",gr,false));
		}
	}

	public boolean action(Event e, Object arg)
	{
	if (e.target == zoom1) dcanvas.setZoom(0.5);
	else if (e.target == zoom2) dcanvas.setZoom(1.0);
	else if (e.target == zoom3) dcanvas.setZoom(2.0);
	else if (e.target == zoom4) dcanvas.setZoom(4.0);
	return true;
	}
}


class DrawingCanvas extends java.awt.Canvas
{
	Drawing drawing;		// the Drawing being displayed
	boolean scrollable;		// can the user scroll with the mouse?
	double zoom = 1.0;		// zoom factor
	Image im;			// backing image
	int w,h;			// size of drawing in pixels
	int xoff = 0, yoff = 0;		// corner of canvas in image
	int xclick, yclick;		// where in image click was

	DrawingCanvas(Drawing dr, boolean sc)
	{
	scrollable = sc;
	setdrawing(dr);
	}

	DrawingCanvas(boolean sc)
	{
	scrollable = sc;
	}

	// setdrawing
	// Set the drawing to scroll to the one given
	void setdrawing(Drawing d)
	{
	drawing = d;
	im = null;
	paint(getGraphics());
	}

	// setZoom
	// Change the zoom factor of this canvas
	public void setZoom(double z)
	{
	zoom = z;
	xoff = yoff = 0;
	im = null;
	paint(getGraphics());
	}

	// renderDrawing
	// Rerender the drawing into a backing image of the right size
	void renderDrawing()
	{
	drawing.scale = zoom;
	drawing.xoffset = 0;
	drawing.yoffset = 0;
	Rectangle drsize = drawing.bounding(getGraphics());
	w = drsize.width; h = drsize.height;
	im = createImage(w, h);
	im.getGraphics().setColor(Color.black);
	im.getGraphics().fillRect(0,0,w,h);
	drawing.xoffset = -drsize.x;
	drawing.yoffset = -drsize.y;
	drawing.render(im.getGraphics());
	}

	// paint
	// Blit the drawing image into the given graphics.
	public void paint(Graphics g)
	{
	if (g == null || drawing == null) {
		// not visible, or no drawing set
		return;
		}
	if (im == null) {
		// Visible, but no backing image
		renderDrawing();
		}
	g.setColor(Color.black);
	g.fillRect(0, 0, size().width, size().height);
	g.drawImage(im, -xoff, -yoff, this);
	}


	public boolean mouseDown(Event e, int x, int y)
	{
	if (im == null) return false;
	xclick = xoff+x;
	yclick = yoff+y;
	return true;
	}

	public boolean mouseDrag(Event e, int x, int y)
	{
	if (!scrollable || im == null)
		return false;
	if (w > size().width) {
		xoff = xclick - x;
		if (xoff < 0) xoff = 0;
		else if (xoff > w-size().width) xoff = w-size().width;
		}
	if (h > size().height) {
		yoff = yclick - y;
		if (yoff < 0) yoff = 0;
		else if (yoff > h-size().height) yoff = h-size().height;
		}
	getGraphics().drawImage(im, -xoff, -yoff, this);
	return true;
	}
}

