// IOComponent
// An AWT component that reads from an input stream while accepting input
// from the user to write to an output stream.
import java.awt.*;
import java.io.*;

public class IOComponent extends Canvas implements Runnable
{
	LineInputStream in;		// where data comes from
	LineOutputStream out;		// where to write keystrokes to
	boolean line_mode = true;	// send a line at a time, with BS
	Graphics g;			// front graphics
	FontMetrics fnm;
	int x,y;			// cursor position
	int w = 80, h = 25;		// size in characters
	int cw, ch;			// size of a character
	int cas, cds;			// ascent and descent of font
	char ca[][] = new char[25][80];	// array of displayed characters
	String line = "";		// line being entered, for line_mode

	IOComponent(LineInputStream i, LineOutputStream o)
	{
	in = i;
	out = o;
	(new Thread(this)).start();
	}

	// run
	// Read and display bytes from the input stream
	public void run()
	{
	while(true) {
		int c = 0;
		try { c = in.read(); }
		catch(IOException e) { c = -1; }
		if (c < 0) break;
		display((char)c);
		}
	}

	// keyDown
	// Called upon a keypress
	public boolean keyDown(Event evt, int k)
	{
	if (line_mode) {
		char kc = (char)k;
		if (kc == 8) {
			// backspace
			if (line.length() != 0 && x != 0) {
				uncursor();
				x--;
				g.setColor(Color.white);
				g.fillRect(x*cw, y*ch, cw, ch);
				g.setColor(Color.black);
				cursor();
				line = line.substring(0, line.length()-1);
				}
			}
		else if (kc == '\n') {
			// send the line
			display(kc);
			out.puts(line);
			line = "";
			}
		else {
			// Add this character to the line
			display(kc);
			line += (char)k;
			}
		}
	else {
		try { out.write(k); }
		catch(IOException e) { }
		}
	return true;
	}

	public void reshape(int nx, int ny, int nw, int nh)
	{
	g = getGraphics();
	g.setColor(Color.black);
	g.setFont(new Font("courier", Font.PLAIN, 10));
	fnm = g.getFontMetrics();
	cw = fnm.charWidth(' ');
	if (cw < 5) cw *= 2;
	ch = fnm.getHeight();
	cas = fnm.getAscent();
	cds = fnm.getDescent();
	w = nw/cw;
	h = nh/ch;
	if (x >= w || y >= h) {
		x = 0;
		y = h-1;
		}
	char nca[][] = new char[h][w];
	for(int i=0; i<h; i++)
		for(int j=0; j<w; j++)
			nca[i][j] = (i<ca.length && j<ca[0].length) ?
					ca[i][j] : ' ';
	ca = nca;
	super.reshape(nx, ny, nw, nh);
	}

	// display
	// Display the given character at the cursor position
	void display(char c)
	{
	uncursor();
	if (c == '\n') {
		// Move down a line
		x = 0; y++;
		if (y == h) {
			// time to scroll up
			y--;
			for(int i=1; i<h; i++)
				ca[i-1] = ca[i];
			ca[y] = new char[w];
			for(int j=0; j<w; j++)
				ca[y][j] = ' ';
			if (g != null) {
				g.copyArea(0, 0, size().width, size().height,
					   0, -ch);
				g.setColor(Color.white);
				g.fillRect(0, (h-1)*ch, size().width, ch*2);
				g.setColor(Color.black);
				}
			}
		}
	else if (c == '\t') {
		// Tab across
		int n = 8-(x%8);
		for(int i=0; i<n; i++)
			display(' ');
		}
	else if (c >= 32) {
		// Just display a character
		ca[y][x] = c;
		if (g != null) {
			g.drawChars(ca[y], x, 1, x*cw, (y+1)*ch-cds);
			}
		x++;
		if (x == w) display('\n');
		}
	cursor();
	}

	// paint
	// Totall redraw the display
	public void paint(Graphics pg)
	{
	g.setColor(Color.white);
	g.fillRect(0, 0, size().width, size().height);
	g.setColor(Color.black);
	for(int i=0; i<h; i++)
		g.drawChars(ca[i], 0, w, 0, (i+1)*ch-cds);
	cursor();
	}

	// cursor
	// Draw the cursor
	void cursor()
	{
	if (g != null)
		g.drawLine(x*cw, y*ch+10, (x+1)*cw, y*ch+10);
	}

	// uncursor
	// Clear the cursor
	void uncursor()
	{
	if (g != null) {
		g.setColor(Color.white);
		g.drawLine(x*cw, y*ch+10, (x+1)*cw, y*ch+10);
		g.setColor(Color.black);
		}
	}
}

