// ShellInterface
// A command-line interface to a JFS server
import java.io.*;

public class ShellInterface
{
	public static void main(String argv[])
	{
	String server = null;
	String username = null;
	String password = null;
	int port = 9888;
	LineOutputStream stdout = new LineOutputStream(System.out);
	LineInputStream stdin = new LineInputStream(System.in);

	// parse args
	for(int i=0; i<argv.length; i++) {
		if (argv[i].equals("-port"))
			port = Integer.parseInt(argv[++i]);
		else if (argv[i].equals("-name"))
			username = argv[++i];
		else if (argv[i].equals("-pass"))
			password = argv[++i];
		else
			server = argv[i];
		}
	if (server == null) {
		stdout.puts(
		 "usage: java ShellInterface <server> [-port number]\n"+
		 "                           [-name username] [-pass password]"
		 );
		System.exit(1);
		}

	// Ask for username and password
	if (username == null) {
		stdout.print("Login: ");
		try { stdout.flush(); }
		catch(IOException e) { }
		try { username = stdin.gets(); }
		catch(IOException e) {
			System.exit(0);
			}
		}
	if (password == null) {
		stdout.print("Password: ");
		try { stdout.flush(); }
		catch(IOException e) { }
		try { password = stdin.gets(); }
		catch(IOException e) {
			System.exit(0);
			}
		}

	// Connect to the server
	JFSclient client = null;
	try { client = new JFSclient(server, port); }
	catch(IOException e) {
		stdout.puts("Failed to connect to server");
		System.exit(0);
		}
	try { client.auth(username, password); }
	catch(RequestException e) {
		stdout.puts("Login failed : "+e.getMessage());
		System.exit(0);
		}

	// Talk away
	JFSShell shell = new JFSShell(client, stdout);
	try {
		while(true) {
			int b = stdin.read();
			shell.write(b);
			}
		}
	catch(IOException e) { }
	client.close();
	}
}

